import com.GameInterface.Game.Shortcut;
import com.GameInterface.Game.ShortcutBase;
import com.GameInterface.Game.ShortcutData;
import com.GameInterface.ProjectUtils;
import com.GameInterface.SpellBase;
import com.GameInterface.SpellData;
import com.Utils.Colors;
import com.GameInterface.DistributedValue;
import com.GameInterface.DistributedValueBase;
import com.Utils.Archive;

// Constants
var SETTINGS_DVALUE:String = "XDem_CDWatch_SettingsOpen";

// Settings listener
var mSettingsListener:DistributedValue;

// Settings
var mConf;

// Warning MC
var mWarn:MovieClip;

// Dummy warning for settings;
var mDummy:MovieClip;

// Icon
var mIcon:MovieClip;

// Settings window
var mSettings:MovieClip;

// List of abilities being actively watched
var mWatch:Array;

// Icon load
var mIconLoader:MovieClipLoader;

// Viper's TIO
var m_VTIOIsLoadedMonitor:DistributedValue;
var VTIOAddonInfo:String = "XDem's CD Watcher|XDem|1.0.0|XDem_CDWatch_SettingsOpen|_root.xdem_cdwatch\\xdem_cdwatch.mIcon";
var mInTopBar:Boolean;

function onLoad()
{
	// Init
	mWatch = [];

	// Load Settings
	loadConfig();
	
	// Icon
	initIcon();
	
	// Settings listener
	DistributedValue.SetDValue(SETTINGS_DVALUE, false);
	mSettingsListener = DistributedValue.Create(SETTINGS_DVALUE);
	mSettingsListener.SignalChanged.Connect(settingsDValueChanged, this);
	
	// Connect to game CD signal
	Shortcut.SignalCooldownTime.Connect( SlotCooldownTime,this );
	
	// Viper's TIO
	mInTopBar = false;
	m_VTIOIsLoadedMonitor = DistributedValue.Create("VTIO_IsLoaded");
	m_VTIOIsLoadedMonitor.SignalChanged.Connect(SlotCheckVTIOIsLoaded, this);
	SlotCheckVTIOIsLoaded();
}

function SlotCheckVTIOIsLoaded() 
{
	if (DistributedValue.GetDValue("VTIO_IsLoaded")) 
	{
		mInTopBar = true;
		DistributedValue.SetDValue("VTIO_RegisterAddon", VTIOAddonInfo);
	}
}

function loadConfig()
{
	// Get Settings
	mConf = {};
	var conf = DistributedValue.GetDValue("XDemCDWatchData", undefined);
	
	if (conf == undefined) // Default settings
	{
		mConf = 
		{
			iconPos: {x:0, y:0},
			iconScale: 100,
			settingsPos: {x:0, y:100},
			warningPos: {x:0, y:400},
			warningScale: 100,
			abilitiesWatched: []
		};
	}
	else
	{
		// Icon Pos
		mConf.iconPos = {};
		mConf.iconPos.x = conf.FindEntry("iconPosX", undefined);
		
		if (mConf.iconPos.x == undefined)
		{
			mConf.iconPos.x = 500;
		}
		
		mConf.iconPos.y = conf.FindEntry("iconPosY", undefined);
		
		if (mConf.iconPos.y == undefined)
		{
			mConf.iconPos.y = 50;
		}
		
		// Icon Scale
		mConf.iconScale = conf.FindEntry("iconScale", undefined);
		
		if (mConf.iconScale == undefined)
		{
			mConf.iconScale = 100;
		}
		
		// Settings Pos
		mConf.settingsPos = {};
		mConf.settingsPos.x = conf.FindEntry("settingsPosX", undefined);
		
		if (mConf.settingsPos.x == undefined)
		{
			mConf.settingsPos.x = 350;
		}
		
		mConf.settingsPos.y = conf.FindEntry("settingsPosY", undefined);
		
		if (mConf.settingsPos.y == undefined)
		{
			mConf.settingsPos.y = 100;
		}
		
		// Warning pos
		mConf.warningPos = {};
		mConf.warningPos.x = conf.FindEntry("warningPosX", undefined);
		
		if (mConf.warningPos.x == undefined)
		{
			mConf.warningPos.x = 350;
		}
		
		mConf.warningPos.y = conf.FindEntry("warningPosY", undefined);
		
		if (mConf.warningPos.y == undefined)
		{
			mConf.warningPos.y = 550;
		}
		
		// Warning Scale
		mConf.warningScale = conf.FindEntry("warningScale", undefined);
		
		if (mConf.warningScale == undefined)
		{
			mConf.warningScale = 100;
		}
		
		// Abilities Watched
		var read = conf.FindEntry("abilitiesWatched", undefined);
		
		if (read == undefined)
		{
			mConf.abilitiesWatched = [];
		}
		else
		{
			mConf.abilitiesWatched = read.split("###");
		}
	}
}

function initIcon()
{
	mIcon = attachMovie("Icon", "mIcon", getNextHighestDepth());
	
	// Pos
	mIcon._x = mConf.iconPos.x;
	mIcon._y = mConf.iconPos.y;
	
	// Scale
	mIcon._xscale = mIcon._yscale = mConf.iconScale;
	
	mIcon.onMouseWheel = function (d)
	{
		var scale = mIcon._xscale;
		scale += 3*d;
		
		if (scale < 50)	scale = 50;
		if (scale > 200) scale = 200;
		
		mIcon._xscale = mIcon._yscale = scale;
		saveArchive();
	};
	
	mIcon.onMousePress = function(id)
	{
		if (mInTopBar || id == 2) // Right click
		{
			toggleSettings();
		}
		else if (id == 1) // Left click
		{
			this.startDrag(false);
		}
	};
	
	mIcon.onMouseRelease = function(id)
	{
		if (id == 1)
		{
			this.stopDrag(false);
			saveArchive();
		}
	};
	
	mIcon.onRelease = function(){};
}

// Settings setter & callback
function toggleSettings()
{
	DistributedValue.SetDValue(SETTINGS_DVALUE, !DistributedSettings.GetDValue(SETTINGS_DVALUE));
}

function settingsDValueChanged()
{
	if (DistributedValue.GetDValue(SETTINGS_DVALUE) == true)
	{
		showSettings();
	}
	else
	{
		hideSettings();
	}
}


function showSettings()
{
	if (mSettings == undefined)
	{
		mSettings = attachMovie("Settings", "mSettings", getNextHighestDepth());
		mSettings.mc_background.onMousePress = function(id)
		{
			if (id == 1)
			{
				mSettings.startDrag(false);
			}
		};
		mSettings.mc_background.onMouseRelease = function(id)
		{
			if (id == 1)
			{
				mSettings.stopDrag(false);
				saveArchive();
			}
		};
		mSettings.mc_background.onRelease = function(){};
		mSettings.bt_apply.onPress = saveSettings;
		mSettings.bt_cancel.onPress = function() { DistributedValue.SetDValue(SETTINGS_DVALUE, false); };
		
		// Pos
		mSettings._x = mConf.settingsPos.x;
		mSettings._y = mConf.settingsPos.y;
		
		// Abilities Watched
		if (mConf.abilitiesWatched.length == 0)
		{
			mSettings.txt_abilities.text = "";
		}
		else
		{
			mSettings.txt_abilities.text = mConf.abilitiesWatched.join(String.fromCharCode(13));
		}
		
		// Dummy warning
		if (mDummy == undefined)
		{
			mDummy = attachMovie("DummyContainer", "mDummy", getNextHighestDepth());
			mDummy._x = mConf.warningPos.x;
			mDummy._y = mConf.warningPos.y;
			mDummy._xscale = mDummy._yscale = mConf.warningScale;
			
			mDummy.onRelease = function (){};
			
			mDummy.onMousePress = function(id)
			{
				if (id == 1)
				{
					this.startDrag(false);
				}
			};
			
			mDummy.onMouseRelease = function(id)
			{
				if (id == 1)
				{
					this.stopDrag(false);
					mConf.warningPos.x = mDummy._x;
					mConf.warningPos.y = mDummy._y;
					saveArchive();
				}
			};
			
			mDummy.onMouseWheel = function(d)
			{
				var scale = mDummy._xscale;
				scale += 5*d;
				
				if (scale < 0)	scale = 0;
				
				mDummy._xscale = mDummy._yscale = scale;
				mConf.warningScale = mDummy._xscale;
				saveArchive();
			}
		}
	}
	else
	{
		mSettings._visible = true;
		mDummy._visible = true;
	}
}

function saveSettings()
{
	mConf.abilitiesWatched = mSettings.txt_abilities.text.split(String.fromCharCode(13));
	saveArchive();
	DistributedValue.SetDValue(SETTINGS_DVALUE, false);
}

function hideSettings()
{
	if (mSettings == undefined)		return;
	
	mSettings._visible = false;
	mDummy._visible = false;
}

// Warning - also called each GCD
function SlotCooldownTime( itemPos:Number, cooldownStart:Number, cooldownEnd:Number,  cooldownFlags:Number ) : Void
{
	var slotID:Number = itemPos - ProjectUtils.GetUint32TweakValue("PlayerStartSlotSpells");
	var seconds:Number = cooldownEnd - cooldownStart;
	
	if (seconds <= 1 && mWatch[slotID] == true)
	{		
		// Get the name from the list
		var list:Object = Shortcut.m_ShortcutList;
		var spell:Object = list[100+slotID];
		
		if (spell != undefined)
		{
			// Show warning
			showWarning(spell);
			
			// Unwatch the ability
			mWatch[slotID] = false;
		}
	}
	else if (seconds > 1) // Don't enter this if we are called from GCD
	{
		// Is it in the list of abilities to watch out for?
		if (NeedWatching(slotID))
		{
			// Mark it as watched
			mWatch[slotID] = true;
		}
	}
}

// Show the ability CD ready warning
function showWarning(spell)
{
	if (mWarn == undefined)
	{
		// Create warning
		mIconLoader = new MovieClipLoader();
		
		mWarn = attachMovie("Container", "mWarn", getNextHighestDepth());
		
		// Hide cooldown line
		mWarn.mc_icon.mc_bg.m_CooldownLine._visible = false;
		
		// This is used to flash when a CD is over - not used here (see ActiveAbility.as)
		mWarn.mc_icon.mc_bg.m_BackgroundOverlay._visible = false;
		mWarn.mc_icon.mc_bg.m_InnerLine._visible = false;
	}
	
	// Pos
	mWarn._x = mConf.warningPos.x;
	mWarn._y = mConf.warningPos.y;
	mWarn._xscale = mWarn._yscale = mConf.warningScale;
	
	// Load icon
	mIconLoader.loadClip(com.GameInterface.Utils.CreateResourceString(spell.m_Icon), mWarn.mc_icon.mc_loader);
	mWarn.mc_icon.mc_loader._xscale = mWarn.mc_icon.mc_loader._yscale = 45;
	
	// Color bg
	var color:Object =  Colors.GetColorlineColors( spell.m_ColorLine );			
	Colors.ApplyColor( mWarn.mc_icon.mc_bg.m_Background.background, color.background );
	Colors.ApplyColor( mWarn.mc_icon.mc_bg.m_Background.highlight, color.highlight );
	
	// Show correct frame
	var spellData:SpellData = SpellBase.GetSpellData(spell.m_SpellId);
	
	if (spellData.m_SpellType == _global.Enums.SpellItemType.eEliteActiveAbility) // elite
	{
		mWarn.mc_icon.mc_bg.m_AuxilliaryFrame._visible = false;
		mWarn.mc_icon.mc_bg.m_EliteFrame._visible = true;
	}
	else if (spellData.m_SpellType == _global.Enums.SpellItemType.eAuxilliaryActiveAbility) // Auxilliary
	{
		mWarn.mc_icon.mc_bg.m_AuxilliaryFrame._visible = true;
		mWarn.mc_icon.mc_bg.m_EliteFrame._visible = false;
	}
	else
	{
		mWarn.mc_icon.mc_bg.m_AuxilliaryFrame._visible = false;
		mWarn.mc_icon.mc_bg.m_EliteFrame._visible = false;
	}
	
	// Show CD ready message
	mWarn._alpha = 100;
	mWarn._visible = true;
	mWarn.warn_txt.text = spell.m_Name;
}

// Remove the ability CD warning
function hideWarning()
{
	if (mWarn != undefined)
	{
		mWarn._visible = false;
	}
}

// Checks if ability is in the list of abilities to watch
function NeedWatching(slotID)
{
	// Get the name from the list
	var list:Object = Shortcut.m_ShortcutList;
	var spell:Object = list[100+slotID];
	
	if (spell != undefined)
	{
		for (var i = 0; i < mConf.abilitiesWatched.length; i++)
		{
			if (mConf.abilitiesWatched[i].toLowerCase() == spell.m_Name.toLowerCase())
			{
				return true;
			}
		}
	}
	
	return false;
}

// Save mod prefs
function saveArchive()
{
	var conf:Archive = new Archive();
	conf.AddEntry("iconPosX", mIcon._x);
	conf.AddEntry("iconPosY", mIcon._y);
	conf.AddEntry("iconScale", mIcon._xscale);
	
	if (mSettings != undefined)
	{
		conf.AddEntry("settingsPosX", mSettings._x);
		conf.AddEntry("settingsPosY", mSettings._y);
	}
	else
	{
		conf.AddEntry("settingsPosX", mConf.settingsPos.x);
		conf.AddEntry("settingsPosY", mConf.settingsPos.y);
	}
	
	if (mWarn != undefined)
	{
		conf.AddEntry("warningPosX", mWarn._x);
		conf.AddEntry("warningPosY", mWarn._y);
		conf.AddEntry("warningScale", mWarn._xscale);
	}
	else
	{
		conf.AddEntry("warningPosX", mConf.warningPos.x);
		conf.AddEntry("warningPosY", mConf.warningPos.y);
		conf.AddEntry("warningScale", mConf.warningScale);
	}
	
	if (mConf.abilitiesWatched.length > 0)
	{
		var abs = mConf.abilitiesWatched.join("###");
		conf.AddEntry("abilitiesWatched", abs);
	}
	
	DistributedValueBase.SetDValue("XDemCDWatchData", conf);
}

function onEnterFrame()
{
	if (mWarn != undefined)
	{
		if (mWarn._alpha > 0)
		{
			mWarn._alpha--;
		}
		else if (mWarn._visible == true)
		{
			hideWarning();
		}
	}
}

function onUnload()
{		
	// Disconnect signals
	Shortcut.SignalCooldownTime.Disconnect( SlotCooldownTime,this );
	
	// Save settings
	saveArchive();
	
	// Remove clips
	mIcon.unloadMovieClip();
	mIcon = undefined;
	
	mSettings.unloadMovieClip();
	mSettings = undefined;
}